@@ -44,10 +44,11 @@ sub _getnamehash {
   my $count = $names->count_names;
   my $visible = $names->get_visible_cite;
   my $dm = Biber::Config->get_dm;
+  my @nps = $dm->get_constant_value('nameparts');
 
   # namehash obeys list truncations but not uniquename
   foreach my $n (@{$names->first_n_names($visible)}) {
-    foreach my $nt ($dm->get_constant_value('nameparts')) {# list type so returns list
+    foreach my $nt (@nps) {# list type so returns list
       if (my $np = $n->get_namepart($nt)) {
         $hashkey .= $np;
       }
@@ -59,7 +60,9 @@ sub _getnamehash {
     $hashkey .= '+';
   }
 
-  $logger->trace("Creating MD5 namehash using '$hashkey'");
+  if ($logger->is_trace()) { # performance shortcut
+    $logger->trace("Creating MD5 namehash using '$hashkey'");
+  }
   # Digest::MD5 can't deal with straight UTF8 so encode it first (via NFC as this is "output")
   return md5_hex(encode_utf8(NFC($hashkey)));
 }
@@ -76,10 +79,11 @@ sub _getnamehash_u {
   my $count = $names->count_names;
   my $visible = $names->get_visible_cite;
   my $dm = Biber::Config->get_dm;
+  my @nps = $dm->get_constant_value('nameparts');
 
   # namehash obeys list truncations
   foreach my $n (@{$names->first_n_names($visible)}) {
-    foreach my $nt ($dm->get_constant_value('nameparts')) {# list type so returns list
+    foreach my $nt (@nps) {# list type so returns list
       if (my $np = $n->get_namepart($nt)) {
         if ($nt eq 'given') {
           if (defined($n->get_uniquename)) {
@@ -103,7 +107,9 @@ sub _getnamehash_u {
     $hashkey .= '+';
   }
 
-  $logger->trace("Creating MD5 namehash_u using '$hashkey'");
+  if ($logger->is_trace()) { # performance shortcut
+    $logger->trace("Creating MD5 namehash_u using '$hashkey'");
+  }
   # Digest::MD5 can't deal with straight UTF8 so encode it first (via NFC as this is "output")
   return md5_hex(encode_utf8(NFC($hashkey)));
 }
@@ -116,9 +122,10 @@ sub _getfullhash {
   my $be = $section->bibentry($citekey);
   my $bee = $be->get_field('entrytype');
   my $dm = Biber::Config->get_dm;
+  my @nps = $dm->get_constant_value('nameparts');
 
   foreach my $n (@{$names->names}) {
-    foreach my $nt ($dm->get_constant_value('nameparts')) {# list type so returns list
+    foreach my $nt (@nps) {# list type so returns list
       if (my $np = $n->get_namepart($nt)) {
         $hashkey .= $np;
       }
@@ -130,7 +137,9 @@ sub _getfullhash {
     $hashkey .= '+'
   }
 
-  $logger->trace("Creating MD5 fullhash using '$hashkey'");
+  if ($logger->is_trace()) { # performance shortcut
+    $logger->trace("Creating MD5 fullhash using '$hashkey'");
+  }
   # Digest::MD5 can't deal with straight UTF8 so encode it first (via NFC as this is "output")
   return md5_hex(encode_utf8(NFC($hashkey)));
 }
@@ -145,14 +154,17 @@ sub _genpnhash {
   my $bee = $be->get_field('entrytype');
   my $hashkey = '';
   my $dm = Biber::Config->get_dm;
+  my @nps = $dm->get_constant_value('nameparts');
 
-  foreach my $nt ($dm->get_constant_value('nameparts')) {# list type so returns list
+  foreach my $nt (@nps) {# list type so returns list
     if (my $np = $n->get_namepart($nt)) {
       $hashkey .= $np;
     }
   }
 
-  $logger->trace("Creating MD5 pnhash using '$hashkey'");
+  if ($logger->is_trace()) { # performance shortcut
+    $logger->trace("Creating MD5 pnhash using '$hashkey'");
+  }
   # Digest::MD5 can't deal with straight UTF8 so encode it first (via NFC as this is "output")
   return md5_hex(encode_utf8(NFC($hashkey)));
 }
@@ -208,7 +220,7 @@ sub _genlabel {
   $LABEL_FINAL = 0; # reset final shortcut
 
   foreach my $labelpart (sort {$a->{order} <=> $b->{order}} @{$labelalphatemplate->{labelelement}}) {
-    my $ret = _labelpart($self, $labelpart->{labelpart}, $citekey);
+    my $ret = _labelpart($self, $labelpart->{labelpart}, $citekey, $secnum, $section, $be);
     $label .= $ret->[0] || '';
     $slabel .= $ret->[1] || '';
     last if $LABEL_FINAL;
@@ -219,10 +231,7 @@ sub _genlabel {
 
 # Disjunctive set of label parts
 sub _labelpart {
-  my ($self, $labelpart, $citekey) = @_;
-  my $secnum = $self->get_current_section;
-  my $section = $self->sections->get_section($secnum);
-  my $be = $section->bibentry($citekey);
+  my ($self, $labelpart, $citekey, $secnum, $section, $be) = @_;
   my $bee = $be->get_field('entrytype');
   my $dm = Biber::Config->get_dm;
   my $maxan = Biber::Config->getblxoption('maxalphanames', $bee, $citekey);
@@ -237,9 +246,9 @@ sub _labelpart {
     }
 
     # Deal with various tests
-    # ifnamecount only uses this label template part if the list it is applied to is a certain
+    # ifnames only uses this label template part if the list it is applied to is a certain
     # length
-    if (my $ic = $part->{ifnamecount}) {
+    if (my $inc = $part->{ifnames}) {
       my $f = $part->{content};
       # resolve labelname
       if ($f eq 'labelname') {
@@ -256,10 +265,26 @@ sub _labelpart {
           $visible_names = $total_names;
         }
 
-        next unless $visible_names == $ic;
+        # Deal with ifnames
+        if ($inc =~ m/^\d+$/) {# just a number
+          next unless $visible_names == $inc;
+        }
+        else {# a range
+          my $incr = parse_range_alt($inc);
+          if (not defined($incr->[0])) {# range -x
+            next unless $visible_names <= $incr->[1];
+          }
+          elsif (not defined($incr->[1])) {# range x-
+            next unless $visible_names >= $incr->[0];
+          }
+          else {# range x-y
+            next unless ($visible_names >= $incr->[0] and
+                         $visible_names <= $incr->[1]);
+          }
+        }
       }
     }
-    my $ret = _dispatch_label($self, $part, $citekey);
+    my $ret = _dispatch_label($self, $part, $citekey, $secnum, $section, $be);
     $lp .= $ret->[0];
     $slp .= $ret->[1];
 
@@ -276,10 +301,7 @@ sub _labelpart {
 
 # Main label dispatch method
 sub _dispatch_label {
-  my ($self, $part, $citekey) = @_;
-  my $secnum = $self->get_current_section;
-  my $section = $self->sections->get_section($secnum);
-  my $be = $section->bibentry($citekey);
+  my ($self, $part, $citekey, $secnum, $section, $be) = @_;
   my $code_ref;
   my $code_args_ref;
   my $lp;
@@ -295,7 +317,7 @@ sub _dispatch_label {
     $code_ref = ${_dispatch_table_label($part->{content}, $dm)}[0];
     $code_args_ref = ${_dispatch_table_label($part->{content}, $dm)}[1];
   }
-  return &{$code_ref}($self, $citekey, $code_args_ref, $part);
+  return &{$code_ref}($self, $citekey, $secnum, $section, $be, $code_args_ref, $part);
 }
 
 
@@ -304,16 +326,13 @@ sub _dispatch_label {
 #########################
 
 sub _label_citekey {
-  my ($self, $citekey, $args, $labelattrs) = @_;
+  my ($self, $citekey, $secnum, $section, $be, $args, $labelattrs) = @_;
   my $k = _process_label_attributes($self, $citekey, $citekey, $labelattrs, $args->[0]);
   return [$k, unescape_label($k)];
 }
 
 sub _label_basic {
-  my ($self, $citekey, $args, $labelattrs) = @_;
-  my $secnum = $self->get_current_section;
-  my $section = $self->sections->get_section($secnum);
-  my $be = $section->bibentry($citekey);
+  my ($self, $citekey, $secnum, $section, $be, $args, $labelattrs) = @_;
   my $e = $args->[0];
   my $f;
   if ($args->[1] and
@@ -334,17 +353,14 @@ sub _label_basic {
 
 # literal string - don't post-process this, there is no point
 sub _label_literal {
-  my ($self, $citekey, $args, $labelattrs) = @_;
+  my ($self, $citekey, $secnum, $section, $be, $args, $labelattrs) = @_;
   my $string = $args->[0];
   return [escape_label(unescape_label($string)), unescape_label($string)];
 }
 
 # names
 sub _label_name {
-  my ($self, $citekey, $args, $labelattrs) = @_;
-  my $secnum = $self->get_current_section;
-  my $section = $self->sections->get_section($secnum);
-  my $be = $section->bibentry($citekey);
+  my ($self, $citekey, $secnum, $section, $be, $args, $labelattrs) = @_;
   my $useprefix = Biber::Config->getblxoption('useprefix', $be->get_field('entrytype'), $citekey);
   my $alphaothers = Biber::Config->getblxoption('alphaothers', $be->get_field('entrytype'));
   my $sortalphaothers = Biber::Config->getblxoption('sortalphaothers', $be->get_field('entrytype'));
@@ -397,8 +413,8 @@ sub _label_name {
     # Use name range override, if any
     my $nr_start;
     my $nr_end;
-    if (exists($labelattrs->{namerange})) {
-      my $nr = parse_range($labelattrs->{namerange});
+    if (exists($labelattrs->{names})) {
+      my $nr = parse_range($labelattrs->{names});
       $nr_start = $nr->[0];
       $nr_end = $nr->[1];
 
@@ -433,6 +449,10 @@ sub _label_name {
         }
       }
       $acc .= _process_label_attributes($self, $citekey, $familynames[$i], $labelattrs, $realname, 'family', $i);
+      # put in names sep, if any
+      if (my $nsep = $labelattrs->{namessep}) {
+        $acc .= $nsep unless ($i == $nr_end-1);
+      }
     }
 
     $sortacc = $acc;
@@ -894,10 +914,7 @@ sub _dispatch_table_sorting {
 
 # Main sorting dispatch method
 sub _dispatch_sorting {
-  my ($self, $sortfield, $citekey, $sortlist, $sortelementattributes) = @_;
-  my $secnum = $self->get_current_section;
-  my $section = $self->sections->get_section($secnum);
-  my $be = $section->bibentry($citekey);
+  my ($self, $sortfield, $citekey, $secnum, $section, $be, $sortlist, $sortelementattributes) = @_;
   my $code_ref;
   my $code_args_ref;
   my $dm = Biber::Config->get_dm;
@@ -918,7 +935,7 @@ sub _dispatch_sorting {
     $code_ref = ${_dispatch_table_sorting($sortfield, $dm)}[0];
     $code_args_ref  = ${_dispatch_table_sorting($sortfield, $dm)}[1];
   }
-  return &{$code_ref}($self, $citekey, $sortlist, $sortelementattributes, $code_args_ref);
+  return &{$code_ref}($self, $citekey, $secnum, $section, $be, $sortlist, $sortelementattributes, $code_args_ref);
 }
 
 # Conjunctive set of sorting sets
@@ -932,7 +949,7 @@ sub _generatesortinfo {
   $BIBER_SORT_FINAL = 0;
   $BIBER_SORT_FINAL = '';
   foreach my $sortset (@{$sortscheme->{spec}}) {
-    my $s = $self->_sortset($sortset, $citekey, $sortlist);
+    my $s = $self->_sortset($sortset, $citekey, $secnum, $section, $be, $sortlist);
     # We have already found a "final" item so if this item returns null,
     # copy in the "final" item string as it's the master key for this entry now
     if ($BIBER_SORT_FINAL and not $BIBER_SORT_NULL) {
@@ -948,7 +965,9 @@ sub _generatesortinfo {
   # for debugging purposes
   my $ss = join($sorting_sep, @$sortobj);
   $sortlist->set_sortdata($citekey, [$ss, $sortobj]);
-  $logger->debug("Sorting object for key '$citekey' -> " . Data::Dump::pp($sortobj));
+  if ($logger->is_debug()) { # performance shortcut
+    $logger->debug("Sorting object for key '$citekey' -> " . Data::Dump::pp($sortobj));
+  }
 
   # Generate sortinit. Skip if there is no sortstring, which is possible in tests
   if ($ss) {
@@ -959,23 +978,18 @@ sub _generatesortinfo {
     $ss =~ s/\A$pre$sorting_sep+//;
     my $init = Unicode::GCString->new(normalise_string($ss))->substr(0, 1)->as_string;
 
-    # Collator for determining primary weight hash for sortinit
-    # Using the global sort locale because we only want the sortinit of the first sorting field
-    # and if this was locally different to the global sorting, something would be very strange.
-    my $Collator = Unicode::Collate::Locale->new(locale => Biber::Config->getoption('sortlocale'), level => 1);
-    my $inithash = md5_hex($Collator->viewSortKey($init));
-    $sortlist->set_sortinitdata_for_key($citekey, $init, $inithash);
+    $sortlist->set_sortinitdata_for_key($citekey, $init);
   }
   return;
 }
 
 # Process sorting set
 sub _sortset {
-  my ($self, $sortset, $citekey, $sortlist) = @_;
+  my ($self, $sortset, $citekey, $secnum, $section, $be, $sortlist) = @_;
   foreach my $sortelement (@$sortset[1..$#$sortset]) {
     my ($sortelementname, $sortelementattributes) = %$sortelement;
     $BIBER_SORT_NULL = 0; # reset this per sortset
-    my $string = $self->_dispatch_sorting($sortelementname, $citekey, $sortlist, $sortelementattributes);
+    my $string = $self->_dispatch_sorting($sortelementname, $citekey, $secnum, $section, $be, $sortlist, $sortelementattributes);
     if ($string) { # sort returns something for this key
       if ($sortset->[0]{final}) {
         # If we encounter a "final" element, we return an empty sort
@@ -999,9 +1013,7 @@ sub _sortset {
 ##############################################
 
 sub _sort_citeorder {
-  my ($self, $citekey, $sortlist, $sortelementattributes) = @_;
-  my $secnum = $self->get_current_section;
-  my $section = $self->sections->get_section($secnum);
+  my ($self, $citekey, $secnum, $section, $be, $sortlist, $sortelementattributes) = @_;
   # Pad the numbers so that they sort with "cmp" properly. Assume here max of
   # a million bib entries. Probably enough ...
   # Allkeys and sorting=none means use bib order which is in orig_order_citekeys
@@ -1027,13 +1039,11 @@ sub _sort_citeorder {
 }
 
 sub _sort_integer {
-  my ($self, $citekey, $sortlist, $sortelementattributes, $args) = @_;
+  my ($self, $citekey, $secnum, $section, $be, $sortlist, $sortelementattributes, $args) = @_;
   my $dmtype = $args->[0]; # get day/month field type
-  my $secnum = $self->get_current_section;
-  my $section = $self->sections->get_section($secnum);
-  my $be = $section->bibentry($citekey);
+  my $bee = $be->get_field('entrytype');
   if (my $field = $be->get_field($dmtype)) {
-    return _process_sort_attributes($field, $sortelementattributes);
+    return _translit($dmtype, $bee, _process_sort_attributes($field, $sortelementattributes));
   }
   else {
     return '';
@@ -1041,15 +1051,13 @@ sub _sort_integer {
 }
 
 sub _sort_editort {
-  my ($self, $citekey, $sortlist, $sortelementattributes, $args) = @_;
+  my ($self, $citekey, $secnum, $section, $be, $sortlist, $sortelementattributes, $args) = @_;
   my $edtypeclass = $args->[0]; # get editor type/class field
-  my $secnum = $self->get_current_section;
-  my $section = $self->sections->get_section($secnum);
-  my $be = $section->bibentry($citekey);
+  my $bee = $be->get_field('entrytype');
   if (Biber::Config->getblxoption('useeditor', $be->get_field('entrytype'), $citekey) and
     $be->get_field($edtypeclass)) {
     my $string = $be->get_field($edtypeclass);
-    return _process_sort_attributes($string, $sortelementattributes);
+    return _translit($edtypeclass, $bee, _process_sort_attributes($string, $sortelementattributes));
   }
   else {
     return '';
@@ -1057,28 +1065,22 @@ sub _sort_editort {
 }
 
 sub _sort_entrykey {
-  my ($self, $citekey, $sortlist, $sortelementattributes) = @_;
+  my ($self, $citekey, $secnum, $section, $be, $sortlist, $sortelementattributes) = @_;
   return _process_sort_attributes($citekey, $sortelementattributes);
 }
 
 sub _sort_labelalpha {
-  my ($self, $citekey, $sortlist, $sortelementattributes, $args) = @_;
-  my $secnum = $self->get_current_section;
-  my $section = $self->sections->get_section($secnum);
-  my $be = $section->bibentry($citekey);
+  my ($self, $citekey, $secnum, $section, $be, $sortlist, $sortelementattributes, $args) = @_;
   my $string = $be->get_field('sortlabelalpha') // '';
   return _process_sort_attributes($string, $sortelementattributes);
 }
 
 sub _sort_labelname {
-  my ($self, $citekey, $sortlist, $sortelementattributes, $args) = @_;
-  my $secnum = $self->get_current_section;
-  my $section = $self->sections->get_section($secnum);
-  my $be = $section->bibentry($citekey);
+  my ($self, $citekey, $secnum, $section, $be, $sortlist, $sortelementattributes, $args) = @_;
   # re-direct to the right sorting routine for the labelname
   if (my $lni = $be->get_labelname_info) {
     # Don't process attributes as they will be processed in the real sub
-    return $self->_dispatch_sorting($lni, $citekey, $sortlist, $sortelementattributes);
+    return $self->_dispatch_sorting($lni, $citekey, $secnum, $section, $be, $sortlist, $sortelementattributes);
   }
   else {
     return '';
@@ -1086,14 +1088,11 @@ sub _sort_labelname {
 }
 
 sub _sort_labeltitle {
-  my ($self, $citekey, $sortlist, $sortelementattributes, $args) = @_;
-  my $secnum = $self->get_current_section;
-  my $section = $self->sections->get_section($secnum);
-  my $be = $section->bibentry($citekey);
+  my ($self, $citekey, $secnum, $section, $be, $sortlist, $sortelementattributes, $args) = @_;
   # re-direct to the right sorting routine for the labeltitle
   if (my $lti = $be->get_labeltitle_info) {
     # Don't process attributes as they will be processed in the real sub
-    return $self->_dispatch_sorting($lti, $citekey, $sortlist, $sortelementattributes);
+    return $self->_dispatch_sorting($lti, $citekey, $secnum, $section, $be, $sortlist, $sortelementattributes);
   }
   else {
     return '';
@@ -1102,16 +1101,13 @@ sub _sort_labeltitle {
 
 sub _sort_labeldate {
   no autovivification;
-  my ($self, $citekey, $sortlist, $sortelementattributes, $args) = @_;
+  my ($self, $citekey, $secnum, $section, $be, $sortlist, $sortelementattributes, $args) = @_;
   my $ldc = $args->[0]; # labeldate component
-  my $secnum = $self->get_current_section;
-  my $section = $self->sections->get_section($secnum);
-  my $be = $section->bibentry($citekey);
   # re-direct to the right sorting routine for the labeldate component
   if (my $ldi = $be->get_labeldate_info) {
     if (my $ldf = $ldi->{field}{$ldc}) {
       # Don't process attributes as they will be processed in the real sub
-      return $self->_dispatch_sorting($ldf, $citekey, $sortlist, $sortelementattributes);
+      return $self->_dispatch_sorting($ldf, $citekey, $secnum, $section, $be, $sortlist, $sortelementattributes);
     }
     elsif (exists($ldi->{string})) { # labelyear fallback string
       return '';
@@ -1125,14 +1121,12 @@ sub _sort_labeldate {
 # This is a meta-sub which uses the optional arguments to the dispatch code
 # It's done to avoid having many repetitions of almost identical sorting code
 sub _sort_list {
-  my ($self, $citekey, $sortlist, $sortelementattributes, $args) = @_;
+  my ($self, $citekey, $secnum, $section, $be, $sortlist, $sortelementattributes, $args) = @_;
   my $list = $args->[0]; # get list field
-  my $secnum = $self->get_current_section;
-  my $section = $self->sections->get_section($secnum);
-  my $be = $section->bibentry($citekey);
+  my $bee = $be->get_field('entrytype');
   if ($be->get_field($list)) {
     my $string = $self->_liststring($citekey, $list);
-    return _process_sort_attributes($string, $sortelementattributes);
+    return _translit($list, $bee, _process_sort_attributes($string, $sortelementattributes));
   }
   else {
     return '';
@@ -1143,14 +1137,12 @@ sub _sort_list {
 # It's done to avoid having many repetitions of almost identical sorting code
 # for literal strings which need normalising
 sub _sort_literal {
-  my ($self, $citekey, $sortlist, $sortelementattributes, $args) = @_;
+  my ($self, $citekey, $secnum, $section, $be, $sortlist, $sortelementattributes, $args) = @_;
   my $literal = $args->[0]; # get actual field
-  my $secnum = $self->get_current_section;
-  my $section = $self->sections->get_section($secnum);
-  my $be = $section->bibentry($citekey);
+  my $bee = $be->get_field('entrytype');
   if (my $field = $be->get_field($literal)) {
     my $string = normalise_string_sort($field, $literal);
-    return _process_sort_attributes($string, $sortelementattributes);
+    return _translit($literal, $bee, _process_sort_attributes($string, $sortelementattributes));
   }
   else {
     return '';
@@ -1161,11 +1153,9 @@ sub _sort_literal {
 # It's done to avoid having many repetitions of almost identical sorting code
 # for the editor roles
 sub _sort_name {
-  my ($self, $citekey, $sortlist, $sortelementattributes, $args) = @_;
+  my ($self, $citekey, $secnum, $section, $be, $sortlist, $sortelementattributes, $args) = @_;
   my $name = $args->[0]; # get name field name
-  my $secnum = $self->get_current_section;
-  my $section = $self->sections->get_section($secnum);
-  my $be = $section->bibentry($citekey);
+  my $bee = $be->get_field('entrytype');
   # If there is a biblatex option which controls the use of this name, check it
   if ($CONFIG_OPTSCOPE_BIBLATEX{"use$name"} and
       not Biber::Config->getblxoption("use$name", $be->get_field('entrytype'), $citekey)) {
@@ -1173,7 +1163,7 @@ sub _sort_name {
     }
   if ($be->get_field($name)) {
     my $string = $self->_namestring($citekey, $name, $sortlist);
-    return _process_sort_attributes($string, $sortelementattributes);
+    return _translit($name, $bee, _process_sort_attributes($string, $sortelementattributes));
   }
   else {
     return '';
@@ -1181,26 +1171,21 @@ sub _sort_name {
 }
 
 sub _sort_presort {
-  my ($self, $citekey, $sortlist, $sortelementattributes) = @_;
-  my $secnum = $self->get_current_section;
-  my $section = $self->sections->get_section($secnum);
-  my $be = $section->bibentry($citekey);
+  my ($self, $citekey, $secnum, $section, $be, $sortlist, $sortelementattributes) = @_;
   my $string = Biber::Config->getblxoption('presort', $be->get_field('entrytype'), $citekey);
   return _process_sort_attributes($string, $sortelementattributes);
 }
 
 sub _sort_sortname {
-  my ($self, $citekey, $sortlist, $sortelementattributes) = @_;
-  my $secnum = $self->get_current_section;
-  my $section = $self->sections->get_section($secnum);
-  my $be = $section->bibentry($citekey);
+  my ($self, $citekey, $secnum, $section, $be, $sortlist, $sortelementattributes) = @_;
+  my $bee = $be->get_field('entrytype');
   my $dm = Biber::Config->get_dm;
 
   # sortname is ignored if no use<name> option is defined - see biblatex manual
   if ($be->get_field('sortname') and
       grep {Biber::Config->getblxoption("use$_", $be->get_field('entrytype'), $citekey)} @{$dm->get_fields_of_type('list', 'name')}) {
     my $string = $self->_namestring($citekey, 'sortname', $sortlist);
-    return _process_sort_attributes($string, $sortelementattributes);
+    return _translit('sortname', $bee, _process_sort_attributes($string, $sortelementattributes));
   }
   else {
     return '';
@@ -1208,7 +1193,7 @@ sub _sort_sortname {
 }
 
 sub _sort_string {
-  my ($self, $citekey, $sortlist, $sortelementattributes, $args) = @_;
+  my ($self, $citekey, $secnum, $section, $be, $sortlist, $sortelementattributes, $args) = @_;
   my $string = $args->[0]; # get literal string
   return _process_sort_attributes($string, $sortelementattributes);
 }
@@ -1396,6 +1381,22 @@ sub _liststring {
   return $str;
 }
 
+# transliterate if requested
+sub _translit {
+  my ($target, $entrytype, $string) = @_;
+  if (my $translits = Biber::Config->getblxoption('translit', $entrytype)) {
+    foreach my $tr (@$translits) {
+      if (lc($tr->{target}) eq '*' or
+          $tr->{target} eq $target or
+          first {$target eq $_} @{$DATAFIELD_SETS{$tr->{target}}}) {
+        return call_transliterator($target, $tr->{from}, $tr->{to}, $string);
+      }
+    }
+  }
+  else {
+    return $string;
+  }
+}
 
 1;
 
